;Author: David Krywult

Include "serializer.bb"

Const WIDTH=800
Const HEIGHT=600

Const ISSERVER = False

Const SENDFREQ = 250 ;Sende alle 250 ms sofern mglich

Const ENABLE_CLIENT_SIMULATION = True

Const ENABLE_FRAME_INDEPENDENCE = True

Graphics(WIDTH, HEIGHT)

Const TNetTextTypeNr = 1
Const NetActionCreate = 1
Const NetActionUpdate = 2
Const NetActionDelete = 3
Type TNetText
	Field x#
	Field y#
	Field xs#
	Field ys#
	Field txt$
End Type
;Format fuer Create: ffffs (Alles wird geschickt)
;Format fuer Delete: ffxxx (Geschwindigkeit wird nicht geschickt,
;							da sich diese nicht aendert. Auch der
;							Text ndert sich nicht, und muss somit
;							nicht geschickt werden.)

Const MAX_TEXTS = 100
Dim texts.TNetText(MAX_TEXTS)
Global textCt = 0

If (ISSERVER) Then
	runServer()
Else
	runClient()
EndIf

Function runServer()
	server% = CreateTCPServer(30000)
	stream% = 0
	While (stream=0)
		Cls
		stream% = AcceptTCPStream(server)
		Text(WIDTH/2, HEIGHT/2, "Waiting for a client", True, True)
		Flip(1)
	Wend
	
	nextSend = MilliSecs()+SENDFREQ
	While (Not KeyHit(1))
		
		Cls
		Text(5, 5, "Press left mouse button", False, False)
		If (MouseHit(1)) Then
			createNetText(MouseX(), MouseY(), Rnd(-20,20), Rnd(-20,20), "Text "+textCt, stream%)
		EndIf

		updateTexts()
		drawTexts()
		
		If (nextSend<MilliSecs()) Then
			sendNetTexts(stream%)
			nextSend = MilliSecs()+SENDFREQ
		EndIf
		
		Flip(0)
	Wend
	End
End Function

Function runClient()
	Cls
	Text(WIDTH/2, HEIGHT/2, "Connecting...", True, True)
	Flip(1)
	stream% = OpenTCPStream("127.0.0.1", 30000)
	If (stream%=0) Then
		Cls
		Text(WIDTH/2, HEIGHT/2, "Connection failed!", True, True)
		Text(WIDTH/2, HEIGHT/2+50, "Press any key", True, True)
		Flip(1)
		WaitKey
		End
	EndIf
	While (Not KeyHit(1))
		Cls
		If (ENABLE_CLIENT_SIMULATION) Then
			updateTexts()
		EndIf
		receiveNetTexts(stream%)
		drawTexts()
		
		Flip(0)
	Wend
	End
End Function

Global lastUpdateMs = -1
Function updateTexts()
	If (ENABLE_FRAME_INDEPENDENCE) Then
		If (lastUpdateMs<>-1) Then
			deltaT# = (MilliSecs()-lastUpdateMs)/1000.0
		Else
			deltaT# = 1.0/60.0
		EndIf
		lastUpdateMs = MilliSecs()
		
		For t.TNetText = Each TNetText
			t\x# = t\x#+t\xs#*deltaT#
			t\y# = t\y#+t\ys#*deltaT#
			
			t\x# = t\x# Mod WIDTH
			t\y# = t\y# Mod HEIGHT
		Next
	Else
		For t.TNetText = Each TNetText
			t\x# = t\x#+t\xs#
			t\y# = t\y#+t\ys#
			
			t\x# = t\x# Mod WIDTH
			t\y# = t\y# Mod HEIGHT
		Next
	EndIf
End Function

Function drawTexts()
	For t.TNetText = Each TNetText
		Text(t\x#, t\y#, t\txt$, True, True)
	Next
End Function

Function createNetText(x#, y#, xs#, ys#, txt$, stream%)
	If (textCt>=MAX_TEXTS) Then Return
	
	Local t.TNetText = New TNetText
	texts(textCt) = t
	t\x# = x#
	t\y# = y#
	t\xs# = xs#
	t\ys# = ys#
	t\txt$ = txt$
	
	WriteByte(stream%, TNetTextTypeNr)
	WriteByte(stream%, NetActionCreate)
	WriteInt(stream, textCt)
	serialize(pointers_getPointer(t), "ffffs", stream%)
	
	textCt = textCt+1
End Function

Function sendNetTexts(stream%)
	For i=0 To textCt-1
		WriteByte(stream%, TNetTextTypeNr)
		WriteByte(stream%, NetActionUpdate)
		WriteInt(stream, i) ;ID
		serialize(pointers_getPointer(texts(i)), "ffxxx", stream%)
	Next
End Function

Function receiveNetTexts(stream%)
	While (ReadAvail(stream%)>0)
		typeNr = ReadByte(stream%)
		action = ReadByte(stream%)
		id     = ReadInt(stream%)
		
		If (typeNr = TNetTextTypeNr) Then
			If (action = NetActionCreate) Then
				If (texts(id)=Null) Then
					texts(id) = New TNetText
				EndIf
				deserialize(pointers_getPointer(texts(id)), "ffffs", stream%)
			ElseIf (action = NetActionUpdate) Then
				If (texts(id)=Null) Then
					texts(id) = New TNetText
				EndIf
				deserialize(pointers_getPointer(texts(id)), "ffxxx", stream%)
			ElseIf (action = NetActionDelete And texts(id)<>Null)
				Delete texts(id)
			EndIf
		EndIf
	Wend
End Function